#!/bin/bash

Init()
{
    echo "Init"
    
    rm -f fulllist
    
#	xterm +sb -title "Caching package list." -e "sudo yum list |tee fulllist" &
	xterm +sb -title "Caching package list." -e "apt-cache dumpavail |awk '/^Package:/{sub (/.*:/, \"\"); package=\$0} /^Description:/ {sub (/.*:/, \"\"); print package\"\t\"\$0}' |tee fulllist"
}

Destroy()
{
    echo "Destroy"
    
    rm -vf fulllist sortedresult *.info *.execs
    
    YUMPID=`ps -ef | awk '/\/usr\/bin\/yum /{print $2}'`
    
    sudo kill -9 $YUMPID
    
#    kdialog --msgbox "YUMPID=$YUMPID"
}

ListPackages()
{
    PACKAGEFILE="$1"

	Widget SearchResults clear

    PACKAGECOUNT="`cat $PACKAGEFILE |wc -l`"

    echo "PACKAGECOUNT=\"$PACKAGECOUNT\""
    
    if [ $PACKAGECOUNT -eq "0" ]
    then
        kdialog --title "Package seach" --sorry "Sorry, no packages."
    fi

    if [ $PACKAGECOUNT -ne "0" ] && ( [ $PACKAGECOUNT -lt 200 ] || kdialog --title "High Package Count" --warningyesno "This is $PACKAGECOUNT packages which will take something like $((PACKAGECOUNT/20))secs to load. You sure?" )
    then
        Widget LoadProgress setMaximum $PACKAGECOUNT

        Widget SearchResults setEnabled 0

        RESULTCOUNT=0
        
        LIST=`< $PACKAGEFILE`
        
        xterm +sb -title "Found $PACKAGECOUNT packages. Getting detailed info." -e "apt-cache show `echo $LIST` |tee details"
        
        sed -i '0,/Package/d' details

	    for RESULT in `cat sortedresult`
	    do
            echo "RESULT=$RESULT"

            sed '/'$RESULT'$/,/^Name/!d;/^Name/d' details >"$RESULT.info"
            
#            SUMMARY="`sed '/^Summary:/!d;s/^Summary:[ \t]*//' "$RESULT.info"`"
            SUMMARY="`sed '/^Description:/!d;s/^Description:[ \t]*//' "$RESULT.info"`"
            LOCATION="`sed '/^Repo/!d;s/^Repo.*:[ \t]*//' "$RESULT.info"`"
            
            SUMMARY="`echo $SUMMARY`"
            LOCATION="`echo $LOCATION`"
            
            echo "SUMMARY=$SUMMARY"
            echo "LOCATION=$LOCATION"

		    Widget SearchResults insertItem "$RESULT\t$LOCATION\t$SUMMARY" 0

            ((RESULTCOUNT++))
            echo "RESULTCOUNT=$RESULTCOUNT"

            Widget LoadProgress setText $RESULTCOUNT
	    done

        Widget SearchResults setEnabled 1
    fi
    
    rm -f "$PACKAGEFILE"
}

DetailedSearch()
{
	SEARCHTEXT="`Widget SearchBox text`"
    
    echo "SEARCHTEXT=\"$SEARCHTEXT\""
    
 	xterm +sb -title "Searching for \"$SEARCHTEXT\"" -e "sudo yum search \"$SEARCHTEXT\" |tee detail"
    
    awk 'NF==3 && $2 ~ /[.-]+/ {sub (/\.[^.]*$/, "", $1); print $1}' detail |sort -u |sed 's/\n//' >sortedresult
    
    rm -f detail
    
    ListPackages sortedresult
}

Search()
{
	SEARCHTEXT="`Widget SearchBox text`"
    
    echo "SEARCHTEXT=\"$SEARCHTEXT\""

    awk '/'"$SEARCHTEXT"'/ {sub (/\.[^.]*$/, "", $1); print $1}' fulllist |sort -u |sed 's/\n//' >sortedresult
    
    ListPackages sortedresult
}

SetPackage()
{
	PACKAGE="`Widget SearchResults selection |awk '{print $1}'`"
	LOCATION="`Widget SearchResults selection |awk '{print $2}'`"
    
    echo "PACKAGE=$PACKAGE"
	
	Widget PackageBox setText "$PACKAGE"
    
    if [ "$LOCATION" == "installed" ]
    then
        Widget InstallButton setText "Select Program"
    else
        Widget InstallButton setText "Install"
    fi
        
}

Info()
{
	PACKAGE=`Widget PackageBox text`
 
    echo "PACKAGE=\"$PACKAGE\""
    
    if [ ! -e "$PACKAGE.info" ]
	then
#	    xterm +sb -title "Getting info on \"$PACKAGE\"" -e "sudo yum info $PACKAGE |tee $PACKAGE.info"
	    xterm +sb -title "Getting info on \"$PACKAGE\"" -e "apt-cache show $PACKAGE |tee $PACKAGE.info"
    
#    	sed -i  '0,/Package/d' "$PACKAGE.info"
	fi
	
	kdialog --title "$PACKAGE" --textbox "$PACKAGE.info" 500 350 &
}

Install()
{
	PACKAGE=`Widget PackageBox text |awk '{print $1}'`
   	LOCATION="`Widget SearchResults selection |awk '{print $2}'`"
        
    echo "PACKAGE=$PACKAGE"
    echo "LOCATION=$LOCATION"
    
    if [ -z "`dpkg -L $PACKAGE`" ]
    then
#	    xterm +sb -title "Installing $PACKAGE" -e 'sudo yum install -y '"$PACKAGE"' |tee result'
	    xterm +sb -title "Installing $PACKAGE" -e 'sudo aptitude install -y '"$PACKAGE"' |tee result'

	    kdialog --title "$PACKAGE" --textbox result 750 1000

        rm -f result
    fi
    
#    rpm -ql $PACKAGE |xargs ls -ld |awk '/-..x..x..x.*\/bin\//{sub (/^[^/]*\//, ""); print "/"$0}' >$PACKAGE.execs
    dpkg -L $PACKAGE |xargs ls -ld |awk '/-..x.*\/bin\//{sub (/^[^/]*\//, ""); print "/"$0}' >$PACKAGE.execs
    
    if [ -n "`< $PACKAGE.execs`" ]
    then
        echo "$PACKAGE" >PACKAGE

        kmdr-executor "RunDialog.kmdr"
    fi
}

InitRunDialog()
{
    PACKAGE="`< PACKAGE`"

    echo "PACKAGE=\"$PACKAGE\""

    rm -f PACKAGE

    Widget Summaries clear
   
    for BINARY in `cat $PACKAGE.execs`
    do
        BINARYNAME=`basename $BINARY`
        
        SUMMARY="`man $BINARYNAME | awk '/^NAME/{getline; sub (/^[^-]*-+[ \t]*|[ \t]*$/, ""); print}'`"
        
        SUMMARY="`echo $SUMMARY`"
        
        echo "BINARY=$BINARY"
        echo "BINARYNAME=$BINARYNAME"
        echo "SUMMARY=$SUMMARY"
        
        Widget Summaries insertItem "$BINARYNAME\t$SUMMARY\t$BINARY" 0
        
    done
    
}

SetBinary()
{
	BINARY=`Widget Summaries selection |awk '{sub (/^[^/]*/, ""); print}'`
    
    echo "BINARY=$BINARY"
	
	Widget RunBox setText "$BINARY"
}

BinaryInfo()
{
    BINARY=`Widget Summaries selection |awk '{print $1}'`
    echo "BINARY=$BINARY"

    man $BINARY |cat >binaryinfo

    if [ -n "`< binaryinfo`" ]
    then	
	    kdialog --title "$BINARY" --textbox binaryinfo 600 350 &
    else
        kdialog --title "$BINARY" --sorry "No more information could be obtained for $BINARY"
    fi
    
    sleep 1

    rm -f binaryinfo
}

RunBinary()
{
    RUNSUMMARY="`Widget Summaries selection`"
    echo "RUNSUMMARY=$RUNSUMMARY"
    
    BINARY=`echo $RUNSUMMARY |awk '{print $1}'`
    RUNBINARY="`Widget RunBox text`"
    
    RUNSUMMARY="`echo $RUNSUMMARY`"
    
    echo "RUNSUMMARY=$RUNSUMMARY"
    echo "RUNBINARY=$RUNBINARY"
    
    if [ -n "$RUNBINARY" ]
    then
        xterm -title "$RUNSUMMARY" -hold +sb -e "$RUNBINARY" |tee "$BINARY.log" &
    fi
}
