PROJECTFOLDER="`Widget ProjectFolder text`"
ISOMOUNT="isomount"
ORIGINALROOTMOUNT="rootfsmount"
MERGEDROOTMOUNT="unionfsmount"

CONFIGURATIONFILE="RepackMyLinux.conf"
PERSISTANT="ProjectFolder CDImageBase MergeFileTree OriginalRoot OriginalRAMDisc UnpackedRAMDisc RepackedRAMDisc NewCDISOImage NewRootFS "

echo "PROJECTFOLDER=$PROJECTFOLDER"
echo "ISOMOUNT=$ISOMOUNT"
echo "ORIGINALROOTMOUNT=$ORIGINALROOTMOUNT"
echo "MERGEDROOTMOUNT=$MERGEDROOTMOUNT"

echo "CONFIGURATIONFILE=$CONFIGURATIONFILE"
echo "PERSISTANT=$PERSISTANT"

Init()
{
    echo "Init"

    for LINE in `< $CONFIGURATIONFILE`
    do
        CONTROL=`echo "$LINE" |awk -F'=' '{print $1}'`
        VALUE=`echo "$LINE" |awk -F'=' '{print $2}'`
    
        echo "CONTROL=$CONTROL"
        echo "VALUE=$VALUE"
        
        Status "$CONTROL=$VALUE"
    
        Widget $CONTROL setText $VALUE
    done
    
    Status "Configuration Loaded"
}

Destroy()
{
    echo "Destroy"

    >$CONFIGURATIONFILE

    for CONTROL in $PERSISTANT
    do
        VALUE="`Widget $CONTROL text`"
    
        echo "$CONTROL=$VALUE" >>$CONFIGURATIONFILE
    done
}

MessageBox()
{
    MESSAGE="$1"
    
    kdialog --title "MyLinux OS Builder" --msgbox "$MESSAGE"
}

Status()
{
	Widget StatusBar setText "$1"
}

Mount()
{
	CDIMAGE="`Widget CDImageBase text`"
    CDIMAGENAME="`basename $CDIMAGE`"

    echo "PROJECTFOLDER=$PROJECTFOLDER"
    echo "ISOMOUNT=$ISOMOUNT"
	echo "CDIMAGE=$CDIMAGE"
	echo "CDIMAGENAME=$CDIMAGENAME"
    
    if [ ! -e "$CDIMAGE" ]
    then
        MessageBox "Need to provide valid CD base"
        exit
    fi
	
	Status "Mounting $CDIMAGENAME..."	
    
	if mkdir -vp $PROJECTFOLDER/$ISOMOUNT && sudo mount -v -o loop -t iso9660 "$CDIMAGE" "$PROJECTFOLDER/$ISOMOUNT"
    then
        Widget ProjectFolder setEnabled 0
        Widget CDImageBase setEnabled 0

        Widget MountButton setEnabled 0
        Widget UnmountButton setEnabled 1
        Widget MergeButton setEnabled 1
        Widget UnmergeButton setEnabled 0

	    Status "$CDIMAGENAME mounted"	
    else
	    Status "$CDIMAGENAME mount failed"	
    fi
}

Unmount()
{
	CDIMAGE="`Widget CDImageBase text`"
    CDIMAGENAME="`basename $CDIMAGE`"
    
	echo "CDIMAGE=$CDIMAGE"
	echo "CDIMAGENAME=$CDIMAGENAME"
    
	Status "Umounting $CDIMAGENAME..."
    
	if sudo umount -v "$PROJECTFOLDER/$ISOMOUNT" && rmdir -v $PROJECTFOLDER/$ISOMOUNT
    then
	    Status "$CDIMAGENAME umounted"   
    else
	    Status "$CDIMAGENAME umount failed"
        exit
    fi
   
    Widget CDImageBase setEnabled 1
    Widget ProjectFolder setEnabled 1

    Widget MergeButton setEnabled 0

    Widget MountButton setEnabled 1
    Widget UnmountButton setEnabled 0
}

Merge()
{
	MERGEFILETREE="`Widget MergeFileTree text`"
	ORIGINALROOT="`Widget OriginalRoot text`"
    
	echo "MERGEFILETREE=$MERGEFILETREE"
	echo "ORIGINALROOT=$ORIGINALROOT"
    
	Status "Merging root filesystem to $MERGEDROOTMOUNT"
    
    if mkdir -vp $PROJECTFOLDER/$ORIGINALROOTMOUNT && sudo mount -vt squashfs -o loop "$PROJECTFOLDER/$ORIGINALROOT" "$PROJECTFOLDER/$ORIGINALROOTMOUNT"
    then
        Status "Original root filesystem mounted"
    else
        Status "Original root filesystem mount failed"
        exit
    fi

    if mkdir -vp $PROJECTFOLDER/$MERGEDROOTMOUNT && sudo mount -vt unionfs -o dirs=$PROJECTFOLDER/$MERGEFILETREE:$PROJECTFOLDER/$ORIGINALROOTMOUNT=ro none $PROJECTFOLDER/$MERGEDROOTMOUNT
    then
        Status "Root filesystem merged"
    else
        Status "Root filesystem merge failed"
        exit
    fi
   
    Widget UnmountButton setEnabled 0
    
    Widget MergeFileTree setEnabled 0
    Widget OriginalRoot setEnabled 0
    
    Widget MergeButton setEnabled 0
    Widget UnmergeButton setEnabled 1
    
	Status "Merged root filesystem to $MERGEDROOTMOUNT"
}

Unmerge()
{
	Status "Releasing root filesystem from $MERGEDROOTMOUNT"

    if sudo umount -v $PROJECTFOLDER/$MERGEDROOTMOUNT && rmdir -v $PROJECTFOLDER/$MERGEDROOTMOUNT
    then
        Status "Root filesystem unmerged"
    else
        Status "Root filesystem unmerge failed"
        exit
    fi

    if sudo umount -v $PROJECTFOLDER/$ORIGINALROOTMOUNT && rmdir -v $PROJECTFOLDER/$ORIGINALROOTMOUNT
    then
        Status "Original root filesystem unmounted"
    else
        Status "Original root filesystem unmount failed"
        exit
    fi

    Widget UnmergeButton setEnabled 0
    Widget MergeButton setEnabled 1
    
    Widget MergeFileTree setEnabled 1
    Widget OriginalRoot setEnabled 1
    
    Widget UnmountButton setEnabled 1
    
	Status "Released root filesystem from $MERGEDROOTMOUNT"
}

Shell()
{
    Widget UnmergeButton setEnabled 0

    sudo mount -vo rbind /dev/ $PROJECTFOLDER/$MERGEDROOTMOUNT/dev/
    sudo mount -vo rbind /proc/ $PROJECTFOLDER/$MERGEDROOTMOUNT/proc/
    sudo mount -vo rbind /sys/ $PROJECTFOLDER/$MERGEDROOTMOUNT/sys/

    sudo cp -af /etc/resolv.conf $PROJECTFOLDER/$MERGEDROOTMOUNT/etc/
    
    xterm -T "Root Shell" -fg white -bg "#00007F" -e "sudo chroot $PROJECTFOLDER/$MERGEDROOTMOUNT"
    
    sudo rm -f $PROJECTFOLDER/$MERGEDROOTMOUNT/etc/resolv.conf

    sudo umount -vl $PROJECTFOLDER/$MERGEDROOTMOUNT/dev $PROJECTFOLDER/$MERGEDROOTMOUNT/proc $PROJECTFOLDER/$MERGEDROOTMOUNT/sys
    
    Widget UnmergeButton setEnabled 1
}

FileManager()
{
    Widget UnmergeButton setEnabled 0

    sudo /usr/kde3/bin/konqueror $PROJECTFOLDER/$MERGEDROOTMOUNT

    Widget UnmergeButton setEnabled 1
}

UnpackRAMDisc()
{
    ORIGINALRAMDISC="`Widget OriginalRAMDisc text`"
    UNPACKEDRAMDISC="`Widget UnpackedRAMDisc text`"

    echo "ORIGINALRAMDISC=$ORIGINALRAMDISC"
    echo "UNPACKEDRAMDISC=$UNPACKEDRAMDISC"
    
    THISUSER=`id -u`
    THISGROUP=`id -u`
    UNZIPPED=$PROJECTFOLDER/initrd.cpio
    
    echo "THISUSER=$THISUSER"
    echo "THISGROUP=$THISGROUP"
    echo "UNZIPPED=$UNZIPPED"

    gunzip -c -f $PROJECTFOLDER/$ORIGINALRAMDISC >$UNZIPPED

    mkdir $PROJECTFOLDER/$UNPACKEDRAMDISC

    pushd $PROJECTFOLDER/$UNPACKEDRAMDISC

    cpio -idv <$UNZIPPED

    popd

    rm -f $UNZIPPED

    sudo chown -R $THISUSER:$THISGROUP $PROJECTFOLDER/$UNPACKEDRAMDISC
}

RAMDiscShell()
{
    UNPACKEDRAMDISC="`Widget UnpackedRAMDisc text`"

    echo "UNPACKEDRAMDISC=$UNPACKEDRAMDISC"
    
    xterm -T "RAM Disc Shell" -fg white -bg "#007F00" -e "sudo chroot $PROJECTFOLDER/$UNPACKEDRAMDISC /bin/sh"
}

RAMDiscFileManager()
{
    UNPACKEDRAMDISC="`Widget UnpackedRAMDisc text`"

    echo "UNPACKEDRAMDISC=$UNPACKEDRAMDISC"
    
    konqueror $PROJECTFOLDER/$UNPACKEDRAMDISC  
}

RepackRAMDisc()
{
    UNPACKEDRAMDISC="`Widget UnpackedRAMDisc text`"
    REPACKEDRAMDISC="`Widget RepackedRAMDisc text`"

    echo "UNPACKEDRAMDISC=$UNPACKEDRAMDISC"
    echo "REPACKEDRAMDISC=$REPACKEDRAMDISC"

    THISUSER=`id -u`
    THISGROUP=`id -u`
    UNZIPPED=$PROJECTFOLDER/initrd.cpio
    
    echo "THISUSER=$THISUSER"
    echo "THISGROUP=$THISGROUP"
    echo "UNZIPPED=$UNZIPPED"
    
    pushd $PROJECTFOLDER/$UNPACKEDRAMDISC

    sudo chown -R root:root .

    find . | cpio --create --format='newc' >$UNZIPPED

    sudo chown -R $THISUSER:$THISGROUP .

    popd

    gzip $UNZIPPED

    mv $UNZIPPED.gz $PROJECTFOLDER/$REPACKEDRAMDISC
}

MakeNewRootFS()
{
	NEWROOTFS="`Widget NewRootFS text`"
    
	echo "NEWROOTFS=$NEWROOTFS"
   
    sudo mksquashfs "$PROJECTFOLDER/$MERGEDROOTMOUNT" "$PROJECTFOLDER/$NEWROOTFS" -noappend # -info
}

MakeNewDisc()
{
    echo "ORIGINALROOTMOUNT=$ORIGINALROOTMOUNT"
    
	NEWCDISOIMAGE="`Widget NewCDISOImage text`"
	ORIGINALROOT="`Widget OriginalRoot text`"
	NEWROOTFS="`Widget NewRootFS text`"
	USEORIGINALFS="`Widget OriginalFS checked`"
	USENEWFS="`Widget NewFS checked`"
    ORIGINALRAMDISC="`Widget OriginalRAMDisc text`"
    REPACKEDRAMDISC="`Widget RepackedRAMDisc text`"
	USEORIGINALRAMDISC="`Widget UseOriginalRAMDisc checked`"
	USENEWRAMDISC="`Widget UseNewRAMDisc checked`"
    
	echo "NEWCDISOIMAGE=$NEWCDISOIMAGE"
	echo "ORIGINALROOT=$ORIGINALROOT"
	echo "NEWROOTFS=$NEWROOTFS"
	echo "USEORIGINALFS=$USEORIGINALFS"
	echo "USENEWFS=$USENEWFS"
	echo "ORIGINALRAMDISC=$ORIGINALRAMDISC"
    echo "REPACKEDRAMDISC=$REPACKEDRAMDISC"
    echo "USEORIGINALRAMDISC=$USEORIGINALRAMDISC"
	echo "USENEWRAMDISC=$USENEWRAMDISC"

    if [ "$USENEWFS" == "true" ]
    then
        ROOTFS=$NEWROOTFS
    else
        ROOTFS=$ORIGINALROOT
    fi
    
    if [ -z "`echo "$ROOTFS" |egrep '^/'`" ]
    then
        ROOTFS=$PROJECTFOLDER/$ROOTFS
    fi

    if [ "$USENEWRAMDISC" == "true" ]
    then
        RAMDISC=$REPACKEDRAMDISC
    else
        RAMDISC=$ORIGINALRAMDISC
    fi
    
    if [ -z "`echo "$RAMDISC" |egrep '^/'`" ]
    then
        RAMDISC=$PROJECTFOLDER/$RAMDISC
    fi
    
    VOLUME_LABEL=`basename $NEWCDISOIMAGE .iso`
    
    BOOTLOADER_PATH="isolinux"
    BOOTLOADER="isolinux.bin"
    BOOTCAT="boot.catalog"

    if [ -e $ISOMOUNT/casper ]
    then
        RAMDISC_PATH="casper"
        RAMDISCFILE="initrd.gz"
        ROFS_PATH="casper"
        COMPRESSED_ROFS="filesystem.squashfs"
    else
        RAMDISC_PATH="isolinux"
        RAMDISCFILE="initrd0.img"
        ROFS_PATH="LiveOS"
        COMPRESSED_ROFS="squashfs.img"
    fi
        
	echo "ROOTFS=$ROOTFS"
	echo "RAMDISC=$RAMDISC"
	echo "VOLUME_LABEL=$VOLUME_LABEL"
	echo "BOOTLOADER_PATH=$BOOTLOADER_PATH"
	echo "BOOTLOADER=$BOOTLOADER"
	echo "BOOTCAT=$BOOTCAT"
	echo "COMPRESSED_ROFS=$COMPRESSED_ROFS"
	echo "RAMDISC_PATH=$RAMDISC_PATH"
	echo "ROFS_PATH=$ROFS_PATH"
	echo "RAMDISCFILE=$RAMDISCFILE"

    sudo cp -v $PROJECTFOLDER/$ISOMOUNT/$BOOTLOADER_PATH/$BOOTLOADER $PROJECTFOLDER/

    sudo mkisofs -R -v \
	    -no-emul-boot \
	    -boot-load-size 4 \
	    -boot-info-table \
	    -graft-points \
	    -b "$BOOTLOADER_PATH/$BOOTLOADER" \
	    -V "$VOLUME_LABEL" \
	    -o "$PROJECTFOLDER/$NEWCDISOIMAGE" \
	    -x "$PROJECTFOLDER/$ISOMOUNT/$BOOTLOADER_PATH/$BOOTLOADER" \
	    -x "$PROJECTFOLDER/$ISOMOUNT/$ROFS_PATH/$COMPRESSED_ROFS" \
	    -x "$PROJECTFOLDER/$ISOMOUNT/$RAMDISC_PATH/$RAMDISCFILE" \
	    -x "$PROJECTFOLDER/$ISOMOUNT/$BOOTCAT" \
	    "/$RAMDISC_PATH/$RAMDISCFILE"="$RAMDISC" \
	    "/$ROFS_PATH/$COMPRESSED_ROFS"="$ROOTFS" \
	    "/$BOOTLOADER_PATH/$BOOTLOADER"="$PROJECTFOLDER/$BOOTLOADER" \
	    "$PROJECTFOLDER/$ISOMOUNT"
}
